# carputils

carputils is a Python framework for generating and running openCARP
examples. carputils also provides functions for automatically comparing
simulation results against known solutions (regression testing).

## Installing

### Docker

A quick way to get started is pulling the [openCARP docker image](https://opencarp.org/download/installation#installation-of-opencarp-docker-containers).

### Linux / macOS
To locally run carputils, first install [openCARP](https://opencarp.org/download/installation#building-opencarp-from-source), [Python3](https://www.python.org/downloads/), and [pip](https://pip.pypa.io/en/stable/installing/).

Clone the carputils repository.
```
git clone https://git.opencarp.org/openCARP/carputils.git
cd carputils
```

#### System Installation
Build the carputils and install it to the system.
This is recommended if you have sudo access, and do not expect to develop the carputils.
```
sudo python3 setup.py install
```

If you do not have sudo access, install it to your home directory.
```
python3 setup.py install --user
```

In this case, if `$HOME/.local/bin` is not included in your `PATH` by default, add it to your .bashrc.
```
export PATH=$PATH:$HOME/.local/bin
```

On the other hand, if you are willing to develop the carputils, manually configuring it as follows is recommended.

#### Development Installation
Install the required python packages using `pip` - preferably into user-space:
```
pip3 install --user -r requirements.txt
```

Add `carputils/bin` to your `PATH`, and `carputils` to your `PYTHONPATH`.
For example, if you clone carputils to `$HOME/install`, add the following lines in your .bashrc:
```
export PATH=$PATH:$HOME/install/carputils/bin
export PYTHONPATH=$PYTHONPATH:$HOME/install/carputils
```

Create a `settings.yaml` file, which is a summary of PATHs and (default) parameters associated with your installation.
```
cusettings settings.yaml
```

Open the file and specify the proper directories for openCARP binary files. 
Notice using the *2 or 4 characters indentation* before the keyword `CPU`.
```
CARP_EXE_DIR:
    CPU: $HOME/install/openCARP/bin
```

Set directories for other openCARP tools if you use them. For example,
```
MESHALYZER_DIR:     $HOME/install/meshalyzer
```

Move `settings.yaml` into the root directory of carputils.

## Contributing
You are here to help? Awesome, feel welcome and read [CONTRIBUTING.md](
https://opencarp.org/community/contribute) to learn how to ask questions
and how to work on something and our process for submitting merge requests to
the openCARP project (including carputils).
All members of our community are expected to follow our code of conduct:
[CODE_OF_CONDUCT.md](https://opencarp.org/community/code-of-conduct).
Please make sure you are welcoming and friendly in all of our spaces.

## Authors
See our list of contributors in
[CONTRIBUTORS.md](https://opencarp.org/community/contributors).

## Dependencies
carputils was built upon the open cardiac electrophysiology simulator
[openCARP](http://www.openCARP.org). The simulator calls as well as input
and output files are tailored for openCARP.

## License
carputils is released under the [Apache Software License](LICENSE.md),
Version 2.0 ("Apache 2.0").
